open Tk

let debug = ref false

let vert wid =
  let newsize = ref 0
  and pending_resize = ref false in
  let rec resize () =
    pending_resize := false;
    if !debug then (Printf.eprintf "Resize %d\n" !newsize; flush stderr);
    Text.configure wid [TextHeight !newsize];
    add_timer 100 (fun () -> Frx_after.idle check);
    ()
  and check () = 
    let first, last = Text.yview_get wid in 
      check1 first last

  and check1 first last =
    let curheight = int_of_string (cget wid CHeight) in
      if !debug then begin
      	 Printf.eprintf "C %d %f %f\n" curheight first last; flush stderr
	 end;
      if first = 0.0 && last = 1.0 then ()
      (* Don't attempt anything if widget is not visible *)
      else if not (Winfo.viewable wid) then begin
        if !debug then (Printf.eprintf "C notviewable\n"; flush stderr);
	(* Try again later *)
      	bind wid [[], Expose] (BindSet ([], fun _ ->
	       bind wid [[], Expose] BindRemove;
	       check()))
        end
      else  begin
	let delta = 
	  if last = 0.0 then 1
	  else 
	    let visible = max 0.1 (last -. first) in
	    max 1 (truncate (float curheight *. (1. -. visible))) in
        newsize := max (curheight + delta) !newsize;
	if !debug then (Printf.eprintf "newsize: %d\n" !newsize; flush stderr);
	if !pending_resize then ()
	else begin
	  pending_resize := true;
	  Frx_after.idle resize
	  end
        end

    and scroll first last =
      if !debug then (Printf.eprintf "V %f %f\n" first last; flush stderr);
      if first = 0.0 && last = 1.0 then ()
      else check1 first last
    in
      scroll, check
